DMFADQ = {debug=false};

local function debug(t)
	if DMFADQ.debug then
		print(t);
	end
end

local function isShapeshifted()
	debug("Checking shapeshift...");
	if select(2, UnitClass("player")) == "DRUID" then
		local b = GetShapeshiftForm(true);
		if b ~= 0 then
			debug("YES!");
			return true;
		end
	end
	debug("No.");
	return false;
end

local function alertShapeshifted(name)
	RaidNotice_AddMessage(RaidBossEmoteFrame, "|cffff0000"..name, ChatTypeInfo["RAID_BOSS_EMOTE"]);
	PlaySound(37666, "master");
end


local addon = DMFADQ;

addon.autoQuests = {
	[37911] = { --37868 follow quest without reward
		name = "The Real Big Race",
		camera = false,
	},
	[37910] = { --33756 is follow quest without reward
		name = "The Real Race",
		camera = false,
	},
	
	[29463] = {
		name = "It's Hammer Time",
		camera = false,
	},
	[29436] = {
		name = "The Humanoid Cannonball",
		camera = false,
		onPreRedeemToken = function()
				local shapeshifted = isShapeshifted();
				if shapeshifted then
					alertShapeshifted("Shapeshifted!");
					CloseGossip();
					return false;
				else 
					return true;
				end
			end
	},
	[29438] = {
		name = "He Shoots, He Scores!",
		camera = true,
	},
	[29434] = {
		name = "Tonk Commander",
		camera = false,
	},
	[36481] = {
		name = "Firebird's Challenge",
		camera = false,
		onPreRedeemToken = function()
				local shapeshifted, name = isShapeshifted();
				if shapeshifted then
					alertShapeshifted(name);
				end
				return true;
			end
	},
	[29455] = {
		name = "Target: Turtle",
		camera = true,
	},
}
addon.npcs = {
	[54601] = {
		name = "mole",
		quest = addon.autoQuests[29463],
	},
	[15303] = {
		name = "cannon",
		quest = addon.autoQuests[29436],
	},
	[14841] = {
		name = "rifle",
		quest = addon.autoQuests[29438],
	},
	[54605] = {
		name = "tonk",
		quest = addon.autoQuests[29434],
	},
	[85546] = {
		name = "fire rings",
		quest = addon.autoQuests[36481],
	},
	[54485] = {
		name= "turtle",
		quest = addon.autoQuests[29455],
	},
	
	--race npcs not needed because they have no gossip but show the quest directly
}

-- Handle automatic quest popup after race completion
-- QUEST_WATCH_LIST_CHANGED does not tigger this right
local interceptTarget = nil;

local function interceptFunction(...) 
	debug("Turn in AutoQuest based on InterceptFunction");
	debug("FuncArgs:")
	debug(...);
	interceptTarget(...); 
	addon.turnInAutoQuest();
end

-- Custom intercept function that forwards the call to the real.
-- Should be no problem but "taints" the call of blizzard, which can (possibly) break things (in battle?)
-- -> only install on darkmoon island
local function installIntercept()
	debug("try Intercept");
	if interceptTarget == nil then
		debug("try install Intercept");
		interceptTarget = AutoQuestPopupTracker_AddPopUp;
		AutoQuestPopupTracker_AddPopUp = interceptFunction;
	end
end

local function removeIntercept()
	debug("try uninstall intercept");
	if interceptTarget ~= nil then
		debug("remove intercept");
		AutoQuestPopupTracker_AddPopUp = interceptTarget;
		interceptTarget = nil;
	end
end



-- General functionality

function addon.onGossipShow()
	local type, zero, serverId, instanceId, zoneUid, npcId = strsplit("-",UnitGUID("npc"));
	npcId = tonumber(npcId);
	
	if addon.npcs[npcId] ~= nil then
		addon.onGossipWithDarkmoonNpc(npcId);
	elseif 57850 == npcId then
		addon.onGossipWithDarkmoonCannonTeleportNpc();
	end
end

function addon.onGossipWithDarkmoonNpc(npcId)
	if GetNumGossipAvailableQuests() == 1 then --quest not accepted -> accept
		local name, level, isTrivial, frequency = GetGossipAvailableQuests();
		if frequency == LE_QUEST_FREQUENCY_DAILY then
			SelectGossipAvailableQuest(1);
		end
	end
	if GetNumGossipActiveQuests() == 1 then --quest active
		local name, level, isTrivial, isComplete = GetGossipActiveQuests();
		
		if isComplete then --if quest done
			SelectGossipActiveQuest(1); --open quest window to turn in
		else
			if addon.npcs[npcId].quest.camera then 
				SetView(2);
			end
			
			Dismount();
			local preRedeemCheck = addon.npcs[npcId].quest.onPreRedeemToken;
			if preRedeemCheck == nil or preRedeemCheck() then
				--automatically redeem game token
				--assume game token is the second option (after tutorial)
				--better solution to be found but probably not
				--might be able to detect the blue color
				SelectGossipOption(2);
			end
		end
	end
end

function addon.onQuestShow()
	local qid = GetQuestID();
	local qlidx = GetQuestLogIndexByID(qid);
	
	if addon.autoQuests[qid] ~= nil then
		local title, level, groupsize, isHeader, isCollaped, isComplete = GetQuestLogTitle(qlidx)
		
		if title == nil then --not accepted yet
			AcceptQuest();
		elseif isComplete then --false when not complete but number when it is?
			if addon.autoQuests[qid].camera then 
				SetView(5);
			end
			GetQuestReward(1);
		end
	end
end

function addon.onGossipWithDarkmoonCannonTeleportNpc()
	--only teleport on active cannon quest
	local qlidx = GetQuestLogIndexByID(29436);
	if qlidx ~= 0 then
		SelectGossipOption(1 , "?" , true)
	end
end

--[[
	For quests that have a progress, like gnolls:
	Manual progress possibility:
		GetQuestLogIndexByID(29463)
		GetNumQuestLeaderBoards(questLogIndex)
		GetQuestLogLeaderBoard(objectiveIndex, questLogIndex)
	Or direct:
		local text, type, _, progressCurr, progressMax = GetQuestObjectiveInfo(29463, 1, false)

]]
local questEndAlreardyNotifed ={};
function addon.onQuestLogUpdate()
	--alert after quest progress is in maximum
	local qidx = 0;
	local q = nil;
	if GetQuestLogIndexByID(29463) > 0 then
		q = 29463;
		qidx = GetQuestLogIndexByID(q);
	elseif GetQuestLogIndexByID(29434) > 0 then
		q = 29434;
		qidx = GetQuestLogIndexByID(q);
	elseif GetQuestLogIndexByID(36481) > 0 then
		q = 36481;
		qidx = GetQuestLogIndexByID(q);
	end

	if q ~= nil and qidx > 0 then
		local text, type, _, progressCurr, progressMax = GetQuestObjectiveInfo(q, 1, false);
		debug(GetQuestObjectiveInfo(q, 1, false));
		if progressCurr == progressMax and questEndAlreardyNotifed[q] == nil then
			RaidNotice_AddMessage(RaidBossEmoteFrame, "DONE!", ChatTypeInfo["RAID_BOSS_EMOTE"]);
			PlaySound(37666, "master");
			VehicleExit();
			questEndAlreardyNotifed[q] = progressCurr;
		end
	end
end

local hadWings = false;
function addon.onBuffChange(unitId)
	if UnitIsUnit(unitId, "player") then
		local buffId = 102116;
		local buffIdx = 1;
		while true do
			name, rank, icon, count, dispelType, duration, expires, caster, isStealable, nameplateShowPersonal, spellID, canApplyAura, isBossDebuff, _, nameplateShowAll, timeMod, value1, value2, value3 = UnitAura("player", buffIdx);
			
			if name == nil then
				if hadWings then
					debug("==> GONE");
					hadWings = false;
				end
				break;
			else
				buffIdx = buffIdx + 1;
				if spellID == buffId then
					if not hadWings then
						debug("==> WINGS!!!");
						SPT:SetScript("OnUpdate", function(self, elapsed) SPT:Pull(self, elapsed) end);
						hadWings = true;
					end
					break;
				end
			end
		end
		
	end
end

GossipFrame:HookScript("OnShow", addon.onGossipShow);
QuestFrame:HookScript("OnShow", addon.onQuestShow);


local frame = CreateFrame("FRAME", "DmfadqFrame");

function addon.turnInAutoQuest() 
	local nQuests = GetNumAutoQuestPopUps();
	for i=1, nQuests do
		local qid = GetAutoQuestPopUp(i);
		if addon.autoQuests[qid] ~= nil then
		
			debug("Turning in " .. (addon.autoQuests[qid].name));
			
			ShowQuestComplete(GetQuestLogIndexByID(qid));
		end
	end
end

-- To detect DarkmoonIsland:
-- On enter form other area: direct query: GetCurrentMapAreaID() == 823
-- On entering through portal: ZONE_CHANGED_NEW_AREA: GetCurrentMapAreaID() == -1
--     But QUEST_WATCH_LIST_CHANGED afterwards: GetCurrentMapAreaID() == 823

frame:RegisterEvent("ZONE_CHANGED_NEW_AREA");
frame:RegisterEvent("PLAYER_ENTERING_WORLD");

local onDarkmoonIsland = false;
local lastMapAreaId = -1;
local function detectOnDarkmoonIsland()
	debug(GetZoneText());
	local mapId = C_Map.GetBestMapForUnit("player");
	debug("C_Map.GetBestMapForUnit(\"player\"): " .. tostring(mapId));
	
	if not onDarkmoonIsland and mapId == 407 then
		onDarkmoonIsland = true;
		addon.onDarkmoonIslandEnter();
	elseif onDarkmoonIsland then
		onDarkmoonIsland = false;
		addon.onDarkmoonIslandLeave();
	end
	lastMapAreaId = mapId;
	debug("DarkmoonIsland: "..tostring(onDarkmoonIsland));
end

local function eventHandler(self, event, arg1, ...)
	--debug("Event:"..tostring(event));
	if event == "ZONE_CHANGED_NEW_AREA" or event == "PLAYER_ALIVE" or event == "PLAYER_ENTERING_WORLD" then
		detectOnDarkmoonIsland();
	elseif event == "QUEST_LOG_UPDATE" then
		addon.onQuestLogUpdate();
	elseif event == "UNIT_AURA" then
		addon.onBuffChange(arg1);
	end
end
frame:SetScript("OnEvent", eventHandler);

function addon.onDarkmoonIslandEnter()
	debug("onDarkmoonIslandEnter()");
	frame:RegisterEvent("QUEST_WATCH_LIST_CHANGED");
	frame:RegisterEvent("QUEST_LOG_UPDATE");
	--frame:RegisterEvent("UNIT_AURA"); disable cammom press alert for now
	installIntercept();
end

function addon.onDarkmoonIslandLeave()
	debug("onDarkmoonIslandLeave()");
	frame:UnregisterEvent("QUEST_WATCH_LIST_CHANGED");
	frame:UnregisterEvent("QUEST_LOG_UPDATE");
	--frame:UnregisterEvent("UNIT_AURA");
	removeIntercept();
end

-- Slash command
SLASH_DMFADQ1 = "/dmfadq";
SlashCmdList["DMFADQ"] = function(msg)
	--local _, _, cmd, args = string.find(msg, "%s?(%w+)%s?(.*)");
	for qid, qinfo in pairs (addon.autoQuests) do 
		print((IsQuestFlaggedCompleted(qid)and"\124cff00ff00"or"\124cffff0000")..(qinfo.name).."\124r");
	end
end
